@@ -5,6 +5,9 @@
   (:import java.util.Date
            org.joda.time.DateTime))
 
+;; visual sanity check on version being used:
+(println "\nTesting clj-time with Clojure" (clojure-version) "\n")
+
 (deftest test-now
   (is (= (date-time 2010 1 1)
          (do-at (date-time 2010 1 1)
@@ -381,7 +384,7 @@
   (is (= 30240 (-> 3 weeks in-minutes)))
   (is (thrown? UnsupportedOperationException (-> 2 months in-minutes)))
   (is (thrown? UnsupportedOperationException (-> 2 years in-minutes))))
- 
+
 (deftest test-period-in-hours
   (is (= 0   (-> 30 seconds in-hours)))
   (is (= 0   (-> 4 minutes in-hours)))
@@ -427,6 +430,18 @@
   (is (= 1 (-> 14 months in-years)))
   (is (= 3 (-> 3 years in-years))))
 
+(deftest test-adjust
+  (let [s (now) e (plus s (hours 3))
+        i (interval s e)]
+    (is (abuts? i (adjust i (hours 3))))
+    (is (abuts? i (adjust i (hours -3))))
+    (is (not (within? i (plus s (hours 4)))))
+    (is (within? (adjust i (hours 3)) (plus s (hours 4))))
+    (is (not (within? i (minus s (hours 2)))))
+    (is (within? (adjust i (hours -3)) (minus s (hours 2))))
+    (is (overlaps? i (adjust i (hours 1))))
+    (is (overlaps? i (adjust i (hours -1))))))
+
 (deftest test-within?
   (let [d1 (date-time 1985)
         d2 (date-time 1986)
@@ -491,8 +506,10 @@
     (is (overlaps? ld1 ld5 ld1 ld5))
     (is (overlaps? ld1 ld5 ld2 ld4))
     (is (overlaps? ld2 ld4 ld1 ld5))
-    (is (overlaps? ld1 ld2 ld2 ld3))
-    (is (overlaps? ld2 ld3 ld1 ld2))
+    ;; these ranges abut and do not overlap -- fixed in #264
+    (is (not (overlaps? ld1 ld2 ld2 ld3)))
+    ;; these ranges abut and do not overlap -- fixed in #264
+    (is (not (overlaps? ld2 ld3 ld1 ld2)))
     (is (not (overlaps? ld1 ld2 ld3 ld4)))
     (is (not (overlaps? ld1 ld3 ld4 ld5)))))
 
@@ -592,6 +609,12 @@
   (is (= 1 (week-number-of-year (date-time 2012 12 31))))
   (is (= 1 (week-number-of-year (date-time 2013 1 1)))))
 
+(deftest test-week-year
+  (is (= 2015 (week-year (date-time 2016 1 3))))
+  (is (= 2016 (week-year (date-time 2016 1 4))))
+  (is (= 2016 (week-year (date-time 2017 1 1))))
+  (is (= 2017 (week-year (date-time 2017 1 2)))))
+
 (deftest test-number-of-days-in-the-month
   (is (= 31 (number-of-days-in-the-month 2012 1)))
   (is (= 31 (number-of-days-in-the-month (date-time 2012 1 3))))
